/* Copyright (c) 1998 Lucent Technologies - All rights reserved. */
#include <conf9libs.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <setjmp.h>
#include <stdio.h>
#include <errno.h>
#include <unistd.h>
#include <fcntl.h>

#ifdef HAVE_MALLOC_H
#include <malloc.h>
#endif /* HAVE_MALLOC_H */
#ifndef HAVE_ANSI_REALLOC
#define realloc(p, s) ((p == 0)? malloc(s) : realloc(p, s))
#endif

typedef unsigned short	Rune;

#ifndef HAVE_ULONG
typedef unsigned long	ulong;
#endif
#ifndef HAVE_USHORT
typedef unsigned short	ushort;
#endif
#ifndef HAVE_UINT
typedef unsigned int	uint;
#endif
#ifndef HAVE_UCHAR
typedef unsigned char	uchar;
#endif
#ifndef HAVE_CADDR_T
typedef char *caddr_t;
#endif

#ifndef HAVE_MEMMOVE
extern	void *memmove(void*, const void*, size_t);
#endif
#ifndef HAVE_REMOVE
#define	remove(v)			unlink(v)
#endif
#ifndef HAVE_STRERROR
extern	char *strerror(int);
#endif
