/* include/conf9libs.h.  Generated automatically by configure.  */
/* include/conf9libs.h.in.  Generated automatically from configure.in by autoheader.  */

/* Define whether <sys/types.h> defines these shorthands */
#define HAVE_ULONG 1
#define HAVE_UINT 1
#define HAVE_USHORT 1
/* #undef HAVE_UCHAR */
#define HAVE_CADDR_T 1

/* Define to empty if the keyword does not work.  */
/* #undef const */

/* Define if you need to in order for stat and other things to work.  */
/* #undef _POSIX_SOURCE */

/* Define to `unsigned' if <sys/types.h> doesn't define.  */
/* #undef size_t */

/* Define if you have the ANSI C header files.  */
#define STDC_HEADERS 1

/* Define if the X Window System is missing or not being used.  */
/* #undef X_DISPLAY_MISSING */

/* Define if realloc() has true ANSI semantics */
#define HAVE_ANSI_REALLOC 1

/* Define if we have Rune support in the print functions */
#define PRINT_RUNES 1

/* Define if you have the memmove function.  */
#define HAVE_MEMMOVE 1

/* Define if you have the strerror function.  */
#define HAVE_STRERROR 1

/* Define if you have the <malloc.h> header file.  */
/* #undef HAVE_MALLOC_H */

/* Define if you have the m library (-lm).  */
#define HAVE_LIBM 1

/* Name of package */
#define PACKAGE "9libs"

/* Version number of package */
#define VERSION "1.0"

